/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.io.File;
import java.awt.Color;

/**
   Use this program to experiment with two-dimensional rotations.
<p>
   Suppose you want to rotate a 2D model around the point (a,b) in
   the model's coordinates, and you want this rotation to happen at
   the point (c,d) in the camera's view coordinates. You would do
   that with the following three transformations.
   <pre>{@code
      position.matrix = Matrix.translate(c, d, 0)
                .times( Matrix.rotateZ(theta) )
                .times( Matrix.translate(-a, -b, 0) );
   }</pre>
   Try to do the following. The bronto's nose is approximately at the
   point (x, y)=(0.0834, 0.8540) in the bronto's model coordinates.
   Rotate the bronto around his nose but have his nose located at
   the point (0, 1.5) in camera's view coordinate system.
*/
public class RotationExample_v3
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Use orthographic projection
      scene.camera.perspective = false;

      // Create a 2D Model object.
      final Model m = new GRSModel(new File("assets/grs/bronto.grs"));

      // Set the model's color.
      ModelShading.setColor(m, Color.black);

      // Add the model to the Scene.
      final Position position = new Position(m);
      scene.positionList.add(position);

      // Create a set of x and y axes.
      final Model axes = new Axes2D(-1, +1, -1, +1, 24, 24);
      ModelShading.setColor(axes, Color.red);
      scene.positionList.add(new Position(axes));

      // Create a framebuffer to render our scene into.
      final int vp_width  = 512;
      final int vp_height = 512;
      final FrameBuffer fb = new FrameBuffer(vp_width, vp_height);

      for (int i = 0; i <= 36; i++)
      {
         // Rotate the center of the bronto around the point (-1, 2).
         position.matrix = Matrix.scale(0.33) // scale the model to fit in the view rectangle
                   .times( Matrix.translate(-1, 2, 0) // now rotate
                   .times( Matrix.rotateZ(10*i) )
                   .times( Matrix.translate(-0.5, -0.5, 0) ) );

         // Render again.
         fb.clearFB(Color.lightGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_RotationExample_v3_a_Frame%02d.ppm", i));
      }

      for (int i = 0; i <= 36; i++)
      {
         // Rotate the bronto's nose around the point (0, 1.5).
         position.matrix = Matrix.scale(0.33) // scale the model to fit in the view rectangle
                   .times( Matrix.translate(0, 1.5, 0) // now rotate
                   .times( Matrix.rotateZ(10*i) )
                   .times( Matrix.translate(-0.0834, -0.8540, 0) ) );

         // Render again.
         fb.clearFB(Color.lightGray.darker());
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_RotationExample_v3_b_Frame%02d.ppm", i));
      }
   }
}
