/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around its corner at the origin.
   <pre>{@code
            y                             y

            |                             |
            |                            /|\
          1 +---------+                / 1+  \
            |         |              /    |    \
            |         |             <     |     >
            |         |              \    |    /
            |         |                \  |  /
            |         |                  \|/
    --------+---------+---> x     --------+--------+-------> x
            |         1                   |        1
            |                             |
            |                             |
            |                             |
   }</pre>
<p>
   See the animation <a href="../Example_01.gif">Example_01.gif</a>.
<p>
   <a href="../Example_01.gif"><img src="../Example_01.gif" alt="Example_01.gif"></a>
<p>
   For the i'th frame of the animation, the square's model matrix is
   <pre>{@code
      M = Rz(6*i)
        = Rz(6*(i-1)) * Rz(6)
        = Rz(6) * Rz(6*(i-1)).
   }</pre>
   Notice how the new rotations can be accumulated on the right or left
   ends of the matrix product.
<p>
   In the code below, the main loop for creating the animation
   frames looks like this.
   <pre>{@code
      for (int i = 0; i < 60; i++)
      {
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_01_Frame%03d.ppm", i));

         square_p.matrix.mult( Matrix.rotateZ(6) );
      }
   }</pre>
   Each frame of the animation rotates the square an additional 6 degrees
   from the previous frame.
<p>
   Notice that this code "accumulates" the rotations in the square's model
   matrix. Here is what we mean by that.
<p>
   Let {@code M} denote the model matrix for the square. Initially,
   {@code M} is the identity matrix.
   <pre>{@code
               M = [ 1  0  0  0 ]
                   [ 0  1  0  0 ]
                   [ 0  0  1  0 ]
                   [ 0  0  0  1 ]
   }</pre>
<p>
   Let {@code Rz(a)} denote the 4x4 homogeneous matrix that rotates the
   xy-plane around the z-axis by angle {@code a} degrees counter-clockwise.
   <pre>{@code
           Rz(a) = [ cos(a * (pi/180))   -sin(a * (pi/180))   0   0 ]
                   [ sin(a * (pi/180))    cos(a * (pi/180))   0   0 ]
                   [       0                    0             1   0 ]
                   [       0                    0             0   1 ]
   }</pre>
<p>
   Let {@code square_p} refer to the {@code Position} object that refers
   to our {@code Model} of the square and its model {@code Matrix}.
<p>
   Each time we execute the line of code
   <pre>{@code
       square_p.matrix.mult( Matrix.rotateZ(6) );
   }</pre>
   we multiply the position's model matrix {@code M} on the RIGHT by the matrix
   {@code Rz(6)} and replace {@code M} by the result. Essentially, this line of code
   would look like this.
   <pre>{@code
          M = M * Rz(6).
   }</pre>
   Before the loop begins, {@code M} is the identity matrix,
   <pre>{@code
          M = I.
   }</pre>
   After one iteration of the loop {@code M} is
   <pre>{@code
          M = I * Rz(6).
   }</pre>
   After two iterations, {@code M} is
   <pre>{@code
          M = I * Rz(6) * Rz(6)
   }</pre>
   but this simplifies to
   <pre>{@code
          M = Rz(12).
   }</pre>
   After {@code i} iterations, {@code M} simplifies to
   <pre>{@code
          M = Rz(6*i).
   }</pre>
   So the model matrix is just accumulating the total rotation
   and each new frame accumulates a new rotation on the right hand
   side of the previous frame's tranformation.
<p>
   Here is the scene tree that this program builds for the i'th frame
   of the animation. There is only one Matrix object in the scene. All
   of the transformation information needs to be in that single matrix.
   Notice how the additional transformation needed by each new frame, a
   rotation by 6 degrees, is accumulated in the position's matrix.
   <pre>{@code
             Scene
            /     \
           /       \
      Camera        List<Position>
                          |
                          |
                      Position
                      /       \
                     /         \
                Matrix         Square
                Rz(6i)
   }</pre>
*/
public class Example_01
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();
      // Since this is a 2D example, we can use an orthographic camera.
      final double right = 2.0;
      final double left = -right;
      final double top = right;
      final double bottom = -top;
      scene.camera.projOrtho(left, right, bottom, top);

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Add the axes to the Scene.
      Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Create a Model of a square.
      Model square = new Model("Example_01");
      square.addVertex(new Vertex(0, 0, 0),
                       new Vertex(1, 0, 0),
                       new Vertex(1, 1, 0),
                       new Vertex(0, 1, 0));
      square.addColor(Color.magenta,
                      Color.green,
                      Color.blue,
                      Color.yellow);
      square.addLineSegment(new LineSegment(0, 1, 0, 1),
                            new LineSegment(1, 2, 1, 2),
                            new LineSegment(2, 3, 2, 3),
                            new LineSegment(3, 0, 3, 0));

      // Add the square to the Scene.
      Position square_p = new Position(square);
      scene.addPosition(square_p);

      // Rotate the square around its corner at the origin.
      for (int i = 0; i < 60; i++)
      {
         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_01_Frame%03d.ppm", i));

         // Accumulate rotations of the square in the model matrix.
         square_p.matrix.mult( Matrix.rotateZ(6) );
      }


      // Here is a way to accumulate the rotations on
      // the left side of the Position's model matrix.
      // Reset the model matrix to the identity matrix.
      square_p.matrix = Matrix.identity();

      // Rotate the square around its corner at the origin.
      for (int i = 0; i < 60; i++)
      {
         // Render again.
         fb.clearFB(Color.darkGray.brighter());
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_01a_Frame%03d.ppm", i));

         // Accumulate rotations of the square in the model matrix.
         square_p.matrix = Matrix.rotateZ(6).times(square_p.matrix);
       //square_p.matrix.multLeft( Matrix.rotateZ(6) );  // another way to this
      }
   }
}
