/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around its lower left-hand corner
   placed at the point (1, 0).
   <pre>{@code
            y                             y

            |                             |
            |                             |       / \
          1 +---------+                  1+     /     \
            |         |                   |   /         \
            |         |                   |  <           >
            |         |                   |   \         /
            |         |                   |     \     /
            |         |                   |       \ /
    --------+---------+---> x     --------+--------+-------> x
            |         1                   |        1
            |                             |
            |                             |
            |                             |
   }</pre>
<p>
   See the animation <a href="../Example_02.gif">Example_02.gif</a>.
<p>
   <a href="../Example_02.gif"><img src="../Example_02.gif" alt="Example_02.gif"></a>
<p>
   For the i'th frame of the animation, the square's model matrix is
   <pre>{@code
      M = T(1, 0, 0) * Rz(6*i)
        = T(1, 0, 0) * Rz(6*(i-1)) * Rz(6).
   }</pre>
   Notice how new rotations can be accumulated on the right end of
   the matrix product.
<p>
   In the code below, the main loop for creating the animation frames
   looks like this.
   <pre>{@code
      // Translate the square's lower left-hand corner to the point (1,0).
      square_p.matrix.mult( Matrix.translate(1, 0, 0) );

      for (int i = 0; i <= 60; i++)
      {
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_02_Frame%02d.ppm", i));

         square_p.matrix.mult( Matrix.rotateZ(6) );
      }
   }</pre>
   As in the previous example, each frame of the animation rotates the
   square another 6 degrees from the previous frame. The difference in
   this example is the single translation of the model before the loop.
   The rotations are being accumulated "after" the translation if you
   read this code from top to bottom. On the other hand, the rotations
   are being accumulated "before" the translation if you read this code
   from the bottom up (or you read the tranformation formula from
   right-to-left).
<p>
   Here is the scene tree that this program builds for the i'th frame
   of the animation. There is only one Matrix object in the scene. All
   of the transformation information needs to be in that single matrix.
   Notice how the additional transformation needed by each new frame, a
   rotation by 6 degrees, is accumulated on the right of the position's
   matrix.
   <pre>{@code
             Scene
            /     \
           /       \
      Camera        List<Position>
                          |
                          |
                      Position
                      /       \
                     /         \
                Matrix         Square
             T(1,0)*Rz(6i)
   }</pre>
*/
public class Example_02
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();
      // Since this is a 2D example, we can use an orthographic camera.
      final double right = 2.0;
      final double left = -right;
      final double top = right;
      final double bottom = -top;
      scene.camera.projOrtho(left, right, bottom, top);

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Add the axes to the Scene.
      Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Create a Model of a square.
      Model square = new Model("Example_02");
      square.addVertex(new Vertex(0, 0, 0),
                       new Vertex(1, 0, 0),
                       new Vertex(1, 1, 0),
                       new Vertex(0, 1, 0));
      square.addColor(Color.magenta,
                      Color.green,
                      Color.blue,
                      Color.yellow);
      square.addLineSegment(new LineSegment(0, 1, 0, 1),
                            new LineSegment(1, 2, 1, 2),
                            new LineSegment(2, 3, 2, 3),
                            new LineSegment(3, 0, 3, 0));

      // Add the square to the Scene.
      Position square_p = new Position(square);
      scene.addPosition(square_p);

      // Translate the square's lower left-hand corner to the point (1,0).
      square_p.matrix.mult( Matrix.translate(1, 0, 0) );

      // Rotate the square around its corner at the origin.
      for (int i = 0; i < 60; i++)
      {
         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_02_Frame%03d.ppm", i));

         // Accumulate rotations of the square in the model matrix.
         square_p.matrix.mult( Matrix.rotateZ(6) );
      }
   }
}
