/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around its center, with its center at the origin.
<p>
   Demonstrate the transformations of the coordinate system.
<p>
   In the last example file, we said that this code
   <pre>{@code
      square_p.matrix = Matrix.identity();
      square_p.mult( Matrix.rotateZ(6*i) );
      square_p.mult( Matrix.translate(-0.5, -0.5, 0) );
   }</pre>
   when read from the top down, should be read as a sequence of
   transformations of the coordinate system. (When it is read from
   the bottom up, it should be read as a sequence of transformations
   of vertices.) This program draws a visualization of the coordinate
   systems being transformed.
<p>
   In this animation, each frame of the last example animation is turned
   into four frames  The first frame represents this line of code
   <pre>{@code
      square_p.matrix = Matrix.identity();
   }</pre>
   which restores the coordinates system to the original one. The second
   frame represents this line of code
   <pre>{@code
      square_p.mult( Matrix.rotateZ(6*i) );
   }</pre>
   which rotates the coordinate system. The third frame represents this
   line of code
   <pre>{@code
      square_p.mult( Matrix.translate(-0.5, -0.5, 0) );
   }</pre>
   which translates the rotated coordinate system. The last frame then
   draws the square model in the last coordinate system.
*/
public class Example_04b
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();
      // Since this is a 2D example, we can use an orthographic camera.
      final double right = 2.0;
      final double left = -right;
      final double top = right;
      final double bottom = -top;
      scene.camera.projOrtho(left, right, bottom, top);

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Add the axes to the Scene.
      Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Create two shorter sets of x and y axes.
      Model sAxes1 = new Axes2D(-0.5, +0.5, -0.5, +0.5, 2, 2);
      Model sAxes2 = new Axes2D(-0.5, +0.5, -0.5, +0.5, 2, 2);
      ModelShading.setColor(sAxes1, new Color(150, 0, 0));
      ModelShading.setColor(sAxes2, new Color(100, 0, 0));
      Position sAxes1_p = new Position(sAxes1);
      Position sAxes2_p = new Position(sAxes2);
      // Add the shorter axes to the Scene.
      scene.addPosition(sAxes1_p, sAxes2_p);

      // Create a Model of a square.
      Model square = new Model("Example_04b");
      square.addVertex(new Vertex(0, 0, 0),
                       new Vertex(1, 0, 0),
                       new Vertex(1, 1, 0),
                       new Vertex(0, 1, 0));
      square.addColor(Color.magenta,
                      Color.green,
                      Color.magenta,
                      Color.green);
      square.addLineSegment(new LineSegment(0, 1, 0, 1),
                            new LineSegment(1, 2, 1, 2),
                            new LineSegment(2, 3, 2, 3),
                            new LineSegment(3, 0, 3, 0));

      // Add the square to the Scene.
      Position square_p = new Position(square);
      scene.addPosition(square_p);

      // Rotate the square around its center, with its center at the origin.
      for (int i = 0; i < 60; i++)
      {
         sAxes1_p.matrix = Matrix.identity();
         sAxes2_p.matrix = Matrix.identity();

         // Reset the model matrix to the identity matrix.
         square_p.matrix = Matrix.identity();

         sAxes1_p.matrix.mult( Matrix.rotateZ(6*i) );

         sAxes2_p.matrix.mult( Matrix.rotateZ(6*i) );
         sAxes2_p.matrix.mult( Matrix.translate(-0.5, -0.5, 0) );

         square_p.matrix.mult( Matrix.rotateZ(6*i) );
         square_p.matrix.mult( Matrix.translate(-0.5, -0.5, 0) );

         sAxes1.visible = false;
         sAxes2.visible = false;
         square.visible = false;
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_04b_Frame%03da.ppm", i));

         sAxes1.visible = true;
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_04b_Frame%03db.ppm", i));

         sAxes2.visible = true;
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_04b_Frame%03dc.ppm", i));

         square.visible = true;
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_04b_Frame%03dd.ppm", i));
      }
   }
}
