/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around its upper right hand corner.
<p>
   Do this by "accumulating" rotations.
<p>
   In the last animation, for each new frame we always "go back" to
   the original coordinate system and then translate, rotate,
   translate it. This example shows that we don't need to go back
   to the original coordinate system. We can "save" the rotated
   coordinate system, then translate it, then draw, then "return" to
   the rotated coordinate system, accumulate another rotation (on the
   right), save it again, translate it again, then draw, etc.
   <pre>{@code
      M = T(1, 1, 0) * Rz(6*i) * T(-1, -1, 0)
        = T(1, 1, 0) * Rz(6*(i-1)) * Rz(6) * T(-1, -1, 0).
          \______________________________/
                save this product
   }</pre>
<p>
   This is not a better way to create the animation since it is harder to
   read. But sometimes this idea of "saving" and "restoring" a coordinates
   system can be useful.
*/
public class Example_05b
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();
      // Since this is a 2D example, we can use an orthographic camera.
      final double right = 2.0;
      final double left = -right;
      final double top = right;
      final double bottom = -top;
      scene.camera.projOrtho(left, right, bottom, top);

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Add the axes to the Scene.
      Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Create a Model of a square.
      Model square = new Model("Example_05b");
      square.addVertex(new Vertex(0, 0, 0),
                       new Vertex(1, 0, 0),
                       new Vertex(1, 1, 0),
                       new Vertex(0, 1, 0));
      square.addColor(Color.magenta,
                      Color.green,
                      Color.magenta,
                      Color.green);
      square.addLineSegment(new LineSegment(0, 1, 0, 1),
                            new LineSegment(1, 2, 1, 2),
                            new LineSegment(2, 3, 2, 3),
                            new LineSegment(3, 0, 3, 0));

      // Add the square to the Scene.
      Position square_p = new Position(square);
      scene.addPosition(square_p);

      //  Rotate the square around its upper right hand corner.
      //  Do this by "accumulating" rotations (on the right).
      square_p.matrix.mult( Matrix.translate(1.0, 1.0, 0) );
      Matrix save = square_p.matrix;
      for (int i = 0; i < 60; i++)
      {
         square_p.matrix.mult( Matrix.translate(-1.0, -1.0, 0) );

         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_Example_05b_Frame%03d.ppm", i));

         square_p.matrix = save;  // restore the saved matrix
         square_p.matrix.mult( Matrix.rotateZ(6) ); // accumulate a rotation
         save = square_p.matrix;  // save again
      }
   }
}
