/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around the point (1, 1) in the
   square's model coordinate system, but with the center of
   rotation at the point (0, 1/2) in the camera's view
   coordinate system.
   <pre>{@code
            y                             y

            |                             |
            |                             |
          1 +---------+                 1 +---------+
            |         |                   |         |
            |         |                   |         |
            |         |                   +         |
            |         |                  /|\        |
            |         |                /  |  \      |
    --------+---------+---> x     ---/----+----\----+---> x
            |         1             <     |     >   1
            |                        \    |    /
            |                          \  |  /
            |                            \|/
            |                             |
   }</pre>
<p>
   See the animation <a href="../Example_08.gif">Example_08.gif</a>.
<p>
   <a href="../Example_08.gif"><img src="../Example_08.gif" alt="Example_08.gif"></a>
<p>
   For the i'th frame of the animation, the square's model matrix is
   <pre>{@code
      M = T(0, 1/2, 0) * Rz(6*i) * T(-1, -1, 0).
   }</pre>
   Notice how new rotations are being accumulated between two translations.
<p>
   In general, if we want to rotate the unit square around the point (a, b)
   in the square's model coordinate system, but with the center of rotation
   at the point (c, d) in the camera's view coordinate system, then in the
   i'th frame of the animation, the square's model matrix is
   <pre>{@code
      M = T(c, d, 0) * Rz(6*i) * T(-a, -b, 0)
   }</pre>
*/
public class Example_08
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();
      // Since this is a 2D example, we can use an orthographic camera.
      final double right = 2.0;
      final double left = -right;
      final double top = right;
      final double bottom = -top;
      scene.camera.projOrtho(left, right, bottom, top);

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Add the axes to the Scene.
      Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Create a Model of a square.
      Model square = new Model("Example_08");
      square.addVertex(new Vertex(0, 0, 0),
                       new Vertex(1, 0, 0),
                       new Vertex(1, 1, 0),
                       new Vertex(0, 1, 0));
      square.addColor(Color.magenta,
                      Color.green,
                      Color.blue,
                      Color.yellow);
      square.addLineSegment(new LineSegment(0, 1, 0, 1),
                            new LineSegment(1, 2, 1, 2),
                            new LineSegment(2, 3, 2, 3),
                            new LineSegment(3, 0, 3, 0));

      // Add the square to the Scene.
      Position square_p = new Position(square);
      scene.addPosition(square_p);

      // Rotate the square around the point (3/4, 3/4) in the
      // square's model coordinate system, but with the center of
      // rotation at the point (1/4, 1/4) in the.camera's view
      // coordinate system.
      for (int i = 0; i < 60; i++)
      {
         // Reset the model matrix to the identity matrix.
         square_p.matrix = Matrix.identity();

         // Rotate the square.
         square_p.matrix.mult( Matrix.translate(0, 0.5, 0) );
         square_p.matrix.mult( Matrix.rotateZ(6*i) );
         square_p.matrix.mult( Matrix.translate(-1, -1, 0) );

         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_08_Frame%03d.ppm", i));
      }
   }
}
